import collections
from django import template
from django.utils import six


register = template.Library()


def _get_item(value, key):
    """ Returns specified item of the collection by key or index.
    Example::

        {{ unread_messages|get_item:obj.pk|default_if_none:0 }}

        {% get_item field|get_item:'__class__' '__name__' as field_type %}
    """
    if isinstance(value, collections.Mapping):
        return value.get(key)
    if isinstance(value, collections.Sequence):
        try:
            return value[key]
        except IndexError:
            return None
    return getattr(value, key, None)

register.filter('get_item', _get_item)
register.assignment_tag(_get_item, name='get_item')


@register.filter
def in_list(value, container):
    """ Returns True if `value` is in the `arg` container or comma separated
    values list.
        Example::

            {% if value|in_list:"a,b,c,d" %}...{% endif %}
            {% if value|in_list:selection %}...{% endif %}
    """
    if container is None:
        return False
    if isinstance(container, six.text_type):
        container = container.split(',')
    if not isinstance(container, collections.Container):
        container = list(container)
    return value in container or six.text_type(value) in container


@register.filter
def split(value, size):
    size = int(size)
    length = len(value)
    for i in xrange(0, length, size):
        yield value[i:i + size]


@register.filter
def partition(value, n):
    """
    Break a list into ``n`` pieces. The last list may be larger than the rest if
    the list doesn't break cleanly. That is::

        >>> l = range(10)

        >>> partition(l, 2)
        [[0, 1, 2, 3, 4], [5, 6, 7, 8, 9]]

        >>> partition(l, 3)
        [[0, 1, 2], [3, 4, 5], [6, 7, 8, 9]]

        >>> partition(l, 4)
        [[0, 1], [2, 3], [4, 5], [6, 7, 8, 9]]

        >>> partition(l, 5)
        [[0, 1], [2, 3], [4, 5], [6, 7], [8, 9]]

    """
    try:
        n = int(n)
        value = list(value)
    except (ValueError, TypeError):
        return [value]
    p = len(value) / n
    return [value[p * i:p * (i + 1)] for i in range(n - 1)] + [value[p * (i + 1):]]


@register.filter
def partition_horizontal(value, n):
    """
    Break a list into ``n`` peices, but "horizontally." That is,
    ``partition_horizontal(range(10), 3)`` gives::

        [[1, 2, 3],
         [4, 5, 6],
         [7, 8, 9],
         [10]]

    Clear as mud?
    """
    try:
        n = int(n)
        value = list(value)
    except (ValueError, TypeError):
        return [value]
    newlists = [list() for i in range(n)]
    for i, val in enumerate(value):
        newlists[i % n].append(val)
    return newlists



import functools
import inspect


class cached_property(object):
    """ Decorator for read-only properties evaluated only once within TTL
    period.

    It can be used to created a cached property like this::

        import random

        # the class containing the property must be a new-style class
        class MyClass(object):

            # create property whose value is cached for ten minutes
            @cached_property(ttl=600)
            def randint(self):
                # will only be evaluated every 10 min. at maximum.
                return random.randint(0, 100)

    The value is cached  in the '_cache' attribute of the object instance that
    has the property getter method wrapped by this decorator. The '_cache'
    attribute value is a dictionary which has a key for every property of the
    object which is wrapped by this decorator. Each entry in the cache is
    created only when the property is accessed for the first time and is a
    two-element tuple with the last computed property value and the last time
    it was updated in seconds since the epoch.

    The default time-to-live (TTL) is 300 seconds (5 minutes). Set the TTL to
    zero for the cached value to never expire.

    To expire a cached property value manually just do::

        del instance._cache[<property name>]

    (c) 2011 Christopher Arndt, MIT License
    """
    def __init__(self, ttl=300):
        self.ttl = ttl

    def __call__(self, fget, doc=None):
        self.fget = fget
        self.__doc__ = doc or fget.__doc__
        self.__name__ = fget.__name__
        self.__module__ = fget.__module__
        return self

    def __get__(self, inst, owner):
        import time
        now = time.time()
        try:
            value, last_update = inst._cache[self.__name__]
            if self.ttl > 0 and now - last_update > self.ttl:
                raise AttributeError
        except (KeyError, AttributeError):
            value = self.fget(inst)
            try:
                cache = inst._cache
            except AttributeError:
                cache = inst._cache = {}
            cache[self.__name__] = (value, now)
        return value


def naive_memoize(function):
    "Standard memoization decorator."
    name = function.__name__
    _name = '_' + name

    def method(self):
        if not hasattr(self, _name):
            value = function(self)
            setattr(self, _name, value)
        return getattr(self, _name)

    def invalidate():
        if hasattr(method, _name):
            delattr(method, _name)

    method.__name__ = function.__name__
    method.__doc__ = function.__doc__
    method._invalidate = invalidate
    return method


def decorator(func):
    """ This decorator can be used to turn simple functions or callable objects
    into well-behaved decorators, so long as the decorators are fairly simple.
    Allows to use decorator either with arguments or not.

    Usage::

        @decorator
        def apply(func, *args, **kw):
            return func(*args, **kw)

        @decorator
        class apply:
            def __init__(self, *args, **kw):
                self.args = args
                self.kw   = kw

            def __call__(self, func):
                return func(*self.args, **self.kw)

        @apply
        def test():
            return 'test'

        @apply(2, 3)
        def test(a, b):
            return a + b

        assert test == 5

    Note. There is only one drawback: wrapper checks its arguments for single
    function or class. To avoid wrong behavior you can use keyword arguments
    instead of positional

    Links
        * http://wiki.python.org/moin/PythonDecoratorLibrary
    """

    def _is_func_arg(*args, **kw):
        return len(args) == 1 and len(kw) == 0 and (
            inspect.isfunction(args[0]) or isinstance(args[0], type))

    if isinstance(func, type):
        def class_wrapper(*args, **kw):
            if _is_func_arg(*args, **kw):
                return func()(*args, **kw)  # create class before usage
            return func(*args, **kw)
        class_wrapper.__name__ = func.__name__
        class_wrapper.__module__ = func.__module__
        return class_wrapper

    @functools.wraps(func)
    def func_wrapper(*args, **kw):
        if _is_func_arg(*args, **kw):
            return func(*args, **kw)

        def functor(userFunc):
            return func(userFunc, *args, **kw)

        return functor

    return func_wrapper


def deprecated(func):
    """ Decorator to be used to mark functions as deprecated.
    It will result in a warning being emitted when the function is used.

    Usage::

        @other_decorators_must_be_upper
        @deprecated
        def some_old_function(x,y):
            return x + y

        class SomeClass:

            @deprecated
            def some_old_method(self, x,y):
                return x + y
    """

    @functools.wraps(func)
    def new_func(*args, **kwargs):
        import warnings
        warnings.warn_explicit("Call to deprecated function %(funcname)s." % {
                'funcname': func.__name__,
            },
            category=DeprecationWarning,
            filename=func.func_code.co_filename,
            lineno=func.func_code.co_firstlineno + 1
        )
        return func(*args, **kwargs)

    return new_func


class memoize(object):
    """ Decorator to cache a function's return value each time it is called.
    If called later with the same arguments, the cached value is returned, and
    not re-evaluated.

    Usage::

         @memoize
         def fibonacci(n):
            if n in (0, 1):
               return n
            return fibonacci(n-1) + fibonacci(n-2)
    """

    def __init__(self, func):
        self.func = func
        self.cache = {}

    def __call__(self, *args):
        try:
            return self.cache[args]
        except KeyError:
            value = self.func(*args)
            self.cache[args] = value
            return value
        except TypeError:
            # uncachable -- for instance, passing a list as an argument.
            # Better to not cache than to blow up entirely.
            return self.func(*args)

    def __repr__(self):
        """ Return the function's docstring. """
        return self.func.__doc__

    def __get__(self, obj, objtype):
        """ Support instance methods. """
        return functools.partial(self.__call__, obj)



from django.contrib.auth.backends import ModelBackend
try:
    from django.contrib.auth import get_user_model
except ImportError:
    import django.contrib.auth.models
    get_user_model = lambda: django.contrib.auth.models.User


class EmailBackend(ModelBackend):
    def authenticate(self, username=None, password=None, **kwargs):
        user_model = get_user_model()
        username_field = getattr(user_model, 'USERNAME_FIELD', 'username')
        if username is None:
            username = kwargs.get(username_field)
        if username is None:
            return
        try:
            user = user_model._default_manager.get(email__iexact=username)
            if user.check_password(password):
                return user
        except (user_model.DoesNotExist, user_model.MultipleObjectsReturned):
            return



import datetime

import django
from django import db
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.shortcuts import _get_queryset
from django.utils import six
from django.utils.text import slugify
try:
    from django.utils import timezone
except ImportError:
    timezone = None
from unidecode import unidecode

from .. import random_in_range


__all__ = [
    'create_slug',
    'get_object_or_none',
    'get_object_dto',
    'make_model_dto',
    'model_uid_generator',
    'reverse_lazy',
    'timestamp_with_timezone',
    'update_object_from_dto',
]


def create_slug(value, model_class=None, field_name=None, queryset=None,
        separator='-', max_length=None, exclude=None):
    """
    This function is mostly based on django snippet #690 (http://goo.gl/RYV5x3)
    """
    if isinstance(model_class, six.string_types):
        def __dummy():
            pass

        if "." in model_class:
            app_label, model_name = model_class.split(".")
        else:
            app_label = __dummy.__module__.split('.')[0]
            model_name = model_class
        from django.db import models
        model_class = models.get_model(app_label, model_name)

    slug = slugify(six.text_type(unidecode(value)))

    # calculate max length
    from django.db.models.base import ModelBase
    if max_length is None and isinstance(model_class, ModelBase):
        if field_name is None:
            field_name = "slug"
        slug_field = model_class._meta.get_field(field_name)
        max_length = slug_field.max_length

    if max_length:
        slug = slug[:max_length]
        slug = slug.strip(separator)

    # make unique slug
    if queryset is None and isinstance(model_class, ModelBase):
        queryset = model_class._default_manager.all()
        if isinstance(exclude, dict):
            queryset = queryset.exclude(**exclude)

    # Find a unique slug. If one matches, add '-2' to the end and try again
    # (then '-3', etc).
    # TODO(sprymak): pass index generator function as an argument.
    # create_slug(name, index_generator = lambda x: x + random.randint(1, 9))
    # index = index_generator(index)
    if queryset is not None:
        index = 2
        original_slug = slug
        while not slug or queryset.filter(**{field_name: slug}):
            slug = original_slug
            end = '%s%s' % (separator, index)
            if max_length and len(slug) + len(end) > max_length:
                slug = slug[:max_length - len(end)]
                slug = slug.strip(separator)
            slug = '%s%s' % (slug, end)
            index += 1

    return slug


def get_max_uid_value(model_class, field_name):
    max_value = None
    max_config_values = getattr(settings, 'MAX_UID_VALUE', {})
    if isinstance(max_config_values, six.integer_types):
        return max_config_values

    default_value = max_config_values.get('*')
    model_name = model_class
    if isinstance(model_name, db.models.base.ModelBase):
        model_name = '.'.join([
            model_name._meta.app_label,
            model_name._meta.object_name])
    if isinstance(field_name, six.string_types):
        max_value = max_config_values.get(
            '.'.join([model_name, field_name]), default_value)
    return max_value


class model_uid_generator(object):
    """
        Usage::

        class Entry(models.Model):
          uid = models.BigIntegerField(unique=True, editable=False,
              default=utils.model_uid_generator("blog.Entry", field_name="uid",
                  max_value=MAX_UID_VALUE))
    """
    def __init__(self, model_class, max_value=None, min_value=None, field_name="pk"):
        self.model_class = model_class
        self.min_value = min_value
        self.max_value = max_value
        self.field_name = field_name
        if self.min_value is None:
            self.min_value = 0
        if self.max_value is None:
            self.max_value = get_max_uid_value(self.model_class, self.field_name)
            if self.max_value is None:
                raise ImproperlyConfigured()

    def get_random_id(self):
        """ Get random integer. """
        return random_in_range(
            min_value=self.min_value, max_value=self.max_value)

    def is_unique(self, value):
        # Assume generated value is unique if it is not a Django model
        # or ID field name is empty.
        if not (self.field_name and self.model_class
                and issubclass(self.model_class, db.models.Model)):
            return True
        try:
            queryset = _get_queryset(self.model_class)
            queryset.get(**{self.field_name: value})
        except self.model_class.DoesNotExist:
            return True
        except db.DatabaseError:
            # catch "no such table" errors while south schema inspecting
            pass
        return False

    def __call__(self):
        if self.model_class is not None and not isinstance(
                self.model_class, (six.string_types, db.models.base.ModelBase)):
            raise TypeError("model_class must be either a string or "
                            "subclass of Model; received %r" % self.model_class)

        if isinstance(self.model_class, six.string_types):
            if "." in self.model_class:
                app_label, model_name = self.model_class.split(".")
            else:
                app_label = model_uid_generator.__module__.split('.')[0]
                model_name = self.model_class
            self.model_class = db.models.get_model(app_label, model_name)
            if self.model_class is None:
                raise ValueError("model_class refers to model '%r' "
                                 "that has not been installed" % self.model_class)

        if not isinstance(self.field_name, six.string_types):
            raise TypeError("field_name must be a string; "
                            "received %r" % self.field_name)

        # calculate max length
        if self.max_value is None and isinstance(self.model_class,
                db.models.base.ModelBase):
            field = self.model_class._meta.get_field(self.field_name)
            max_length = field.max_length
            # TODO: guess maximum allowed value based on field type

        val = self.get_random_id()
        while not self.is_unique(val):
            val = self.get_random_id()
        return val


def get_object_or_none(model_class, *args, **kwargs):
    """Utility function just list django.shortcuts.get_object_or_404

    Instead of raising a 404 error this function returns None.

    See: django.shortcuts.get_object_or_404
    """
    queryset = _get_queryset(model_class)
    try:
        return queryset.get(*args, **kwargs)
    except queryset.model.DoesNotExist:
        return None


def make_model_dto(model, **data):
    """ Makes a data transfer object for Django ``db.Model`` class.

        dto = make_model_dto(models.Post, title="Lorem Ipsum")
        print dto.title
        dto.more = 11
        print dto.more
    """

    class final(type):
        """ Metaclass to prevent a class from being inherited """

        def __init__(cls, name, bases, namespace):
            super(final, cls).__init__(name, bases, namespace)
            for klass in bases:
                if isinstance(klass, final):
                    raise TypeError(str(klass.__name__) + " is final")

    class DataTransferObject(object):
        __metaclass__ = final

        def __init__(self, **kwargs):
            self.__dict__ = kwargs

        def __iter__(self):
            return self.__dict__

        def as_dict(self):
            return self.__dict__

    fields = {}
    for field in model._meta.get_all_field_names():
        fields[field] = data.get(field)

    return DataTransferObject(**fields)


def get_object_dto(instance):
    """ Constructs the composite transfer object from Django ``Model`` object.
        :param instance: source model object
    """
    data = {}
    if isinstance(instance, django.db.models.Model):
        from django.core import serializers
        serializer = serializers.get_serializer("python")()
        serializer.use_natural_keys = False

        serializer.start_serialization()
        serializer.start_object(instance)
        for field in instance._meta.local_fields:
            if field.serialize:
                if field.rel is None:
                    serializer.handle_field(instance, field)
                else:
                    try:
                        serializer.handle_fk_field(instance, field)
                    except field.rel.to.DoesNotExist:
                        # skip not found relatives
                        pass
        for field in instance._meta.many_to_many:
            if field.serialize:
                serializer.handle_m2m_field(instance, field)
        serializer.end_object(instance)
        serializer.end_serialization()
        value = serializer.getvalue()
        if value:
            data = value[0].get('fields', {})
    return make_model_dto(instance.__class__, **data)


def update_object_from_dto(instance, data, partial=False):
    model = instance.__class__
    for field in model._meta.get_all_field_names():
        value = getattr(data, field)
        if value or not partial:
            setattr(instance, field, value)


def render_to_json_response(data, **kwargs):
    """ Render data as JSON.

        :param encoder: custom `JSONEncoder` subclass to serialize additional
            types. This argument is an alias to `cls` argument for `json.dump`.
        :param kwargs: arguments suitable to pass to `json.dump` function.
        :returns: `HttpResponse` object with JSON mime type.

        Usage::

            render_to_json_response({'foo':'bar'})
            render_to_json_response({'foo':'bar'}, indent=4)
    """
    from django.http import HttpResponse
    from django.utils import simplejson
    from .decorators import _JSON_MIME_TYPE
    status_code = kwargs.pop('status', None)
    content_type = kwargs.pop('content_type', None)

    if isinstance(data, HttpResponse):
        data.mimetype = _JSON_MIME_TYPE
        if status_code:
            data.status_code = status_code
        if content_type:
            data['Content-Type'] = content_type
        return data

    retval = HttpResponse(mimetype=_JSON_MIME_TYPE, content_type=content_type)
    if status_code:
        retval.status_code = status_code
    encoder = kwargs.pop('encoder', None)
    retval.write(simplejson.dumps(data, cls=encoder, **kwargs))
    return retval


def timestamp_with_timezone(dt=None):
    """ Return a timestamp with a timezone for the configured locale.
    If all else fails, consider localtime to be UTC.
    """
    dt = dt or datetime.datetime.now()
    if timezone is None:
        return dt.strftime('%Y-%m-%d %H:%M%z')
    if not dt.tzinfo:
        tz = timezone.get_current_timezone()
        if not tz:
            tz = timezone.utc
        dt = dt.replace(tzinfo=timezone.get_current_timezone())
    return dt.strftime("%Y-%m-%d %H:%M%z")


# A lazily evaluated version of reverse().
# It is useful for when you need to use a URL reversal before your
# project's URLConf is loaded.
if django.VERSION < (1, 4, None):
    from django.core.urlresolvers import reverse
    from django.utils.functional import lazy
    reverse_lazy = lazy(reverse, str)
else:
    from django.core.urlresolvers import reverse_lazy


def url_name(request):
    """ Returns URL name for current request.
    Origin: https://code.djangoproject.com/ticket/18584

        Usage::

            if url_name(request) != 'auth_login':
                ...
    """
    from django.core.urlresolvers import resolve
    try:
        res = resolve(request.path)
        if res:
            return res.url_name
    except:
        return



""" sendmail email backend class. """
import threading

from django.core.mail.backends.base import BaseEmailBackend
from subprocess import Popen, PIPE


class EmailBackend(BaseEmailBackend):
    def __init__(self, fail_silently=False, **kwargs):
        super(EmailBackend, self).__init__(fail_silently=fail_silently)
        self._lock = threading.RLock()

    def open(self):
        return True

    def close(self):
        pass

    def send_messages(self, email_messages):
        """ Sends one or more EmailMessage objects and returns the number of
            email messages sent.
        """
        if not email_messages:
            return
        self._lock.acquire()
        try:
            num_sent = 0
            for message in email_messages:
                sent = self._send(message)
                if sent:
                    num_sent += 1
        finally:
            self._lock.release()
        return num_sent

    def _send(self, email_message):
        """ A helper method that does the actual sending. """
        if not email_message.recipients():
            return False
        try:
            ps = Popen(["sendmail"] + list(email_message.recipients()),
                stdin=PIPE)
            ps.stdin.write(email_message.message().as_string())
            ps.stdin.flush()
            ps.stdin.close()
            return not ps.wait()
        except:
            if not self.fail_silently:
                raise
            return False
        return True



import re
import collections

from django.http import QueryDict
from django.template import Library, Node, TemplateSyntaxError
from django.utils import six
from django.utils.datastructures import MultiValueDict
import django.utils.http

register = Library()


class UrlParamsNode(Node):
    def __init__(self, url, args, kwargs, asvar):
        self.url = url
        self.args = args
        self.kwargs = kwargs
        self.asvar = asvar

    def render(self, context):
        url = self.url.resolve(context)
        parts = url.split('?')
        url = parts[0]
        original_query = QueryDict('&'.join(parts[1:])).copy()
        query = MultiValueDict(dict(
            (k, v.resolve(context)) for k, v in six.iteritems(self.kwargs)))

        args_to_set = []
        args_to_remove = []
        for arg in self.args:
            k = arg.resolve(context)
            if k.startswith('-'):
                args_to_remove.append(k[1:])
            else:
                args_to_set.append(k)

        kwargs_to_set = MultiValueDict()
        kwargs_to_add = MultiValueDict()
        kwargs_to_remove = MultiValueDict()
        for (k, v) in query.lists():
            # make sure val is a list of strings
            val = map(str, v if isinstance(v, list) else [v])
            if k.startswith('+'):
                kwargs_to_add.setlistdefault(k[1:]).extend(val)
            elif k.startswith('-'):
                kwargs_to_remove.setlistdefault(k[1:]).extend(val)
            else:
                kwargs_to_set.setlist(k, val)

        filter_excluded_kwargs = lambda values: [
            (k, list(set(v) - set(kwargs_to_remove.getlist(k))))
            for k, v in values.iterlists() if k not in args_to_remove]

        args = [arg for arg in args_to_set if arg not in args_to_remove]
        kwargs = MultiValueDict()
        for k, v in filter_excluded_kwargs(original_query):
            kwargs.setlistdefault(k).extend(v if isinstance(v, list) else [v])
        for k, v in filter_excluded_kwargs(kwargs_to_set):
            kwargs.setlist(k, v)
        for k, v in filter_excluded_kwargs(kwargs_to_add):
            kwargs.setlistdefault(k).extend(v)

        output = list(set(args))
        for k, l in kwargs.lists():
            output.extend(
                ['%s=%s' % (k, v) if v else six.text_type(k) for v in l])

        if output:
            url = '?'.join([url, '&'.join(set(output))])

        if self.asvar:
            context[self.asvar] = url
            return ''
        else:
            return url


kwarg_re = re.compile(r"(?:(^[+-]?\w+)=)?(.+)")


@register.filter('base36_to_int')
def do_base36_to_int(value):
    return django.utils.http.base36_to_int(value)


@register.filter('int_to_base36')
def do_int_to_base36(value):
    return django.utils.http.int_to_base36(int(value))


@register.assignment_tag(takes_context=True)
def url_name(context):
    """ Returns URL name of the current request.

    Example::

        {% url_name as current_url_name %}
        {% if current_url_name != 'auth_login' %}
        <li>
        <a href="{% url 'auth_login_next' request.get_full_path|urlencode %}">
            {% trans 'Signin' %}</a>
        </li>
        {% endif %}
    """
    request = context.get("request")
    if not request:
        return ""
    from .. import url_name as _url_name
    return _url_name(request)


@register.tag
def url_params(parser, token):
    """ Changes GET parameters of the URL.

        Set 'mode' parameter to 'list', remove value 9 of 'limit' parameter of
        the current URL::

            {% url_params request.get_full_path mode='list' -limit=9 as url %}

        Add 'list' value to 'mode' parameter, remove all 'limit' values of
        the current URL::

            {% url_params request.get_full_path +mode='list' '-limit' %}
    """
    bits = token.split_contents()
    url = parser.compile_filter(bits[1])
    args = []
    kwargs = {}
    asvar = None
    bits = bits[2:]
    if len(bits) >= 2 and bits[-2] == 'as':
        asvar = bits[-1]
        bits = bits[:-2]
    for bit in bits:
        match = kwarg_re.match(bit)
        if not match:
            raise TemplateSyntaxError("Malformed arguments to url_params tag")
        name, value = match.groups()
        if name:
            kwargs[name] = parser.compile_filter(value)
        else:
            args.append(parser.compile_filter(value))

    return UrlParamsNode(url, args, kwargs, asvar)



"""
python-ldap

### ACTIVE DIRECTORY SETTINGS

# AD_DNS_NAME should set to the AD DNS name of the domain (ie; example.com)
# If you are not using the AD server as your DNS, it can also be set to
# FQDN or IP of the AD server.

AD_DNS_NAME = 'example.com'
AD_LDAP_PORT = 389

AD_SEARCH_DN = 'CN=Users,dc=example,dc=com'

# This is the NT4/Samba domain name
AD_NT4_DOMAIN = 'EXAMPLE'

AD_SEARCH_FIELDS = ['mail', 'givenName', 'sn', 'sAMAccountName']

AD_LDAP_URL = 'ldap://%s:%s' % (AD_DNS_NAME, AD_LDAP_PORT)
"""
import logging
from django.contrib.auth.models import User
from django.conf import settings
try:
    import ldap
except ImportError:
    ldap = None


AD_DNS_NAME = getattr(settings, 'AD_DNS_NAME', '')
AD_LDAP_PORT = getattr(settings, 'AD_LDAP_PORT', 389)
DFAULT_AD_SEARCH_DN = 'CN=Users' + ',dc='.join(('.' + AD_DNS_NAME).split('.'))
AD_NT4_DOMAIN = getattr(settings, 'AD_NT4_DOMAIN',
        AD_DNS_NAME.split('.')[0].upper())
AD_SEARCH_DN = getattr(settings, 'AD_SEARCH_DN', DFAULT_AD_SEARCH_DN)
AD_SEARCH_FIELDS = getattr(settings, 'AD_SEARCH_FIELDS',
        ['mail', 'givenName', 'sn', 'sAMAccountName'])
AD_LDAP_URL = getattr(settings, 'AD_LDAP_URL',
        'ldap://%s:%s' % (AD_DNS_NAME, AD_LDAP_PORT))

logger = logging.getLogger()


class ActiveDirectoryBackend:

    def authenticate(self, username=None, password=None):
        logger.debug('trying to authenticate \"%s\" via Active Directory',
                username)
        if not self.is_valid(username, password):
            return None

        logger.warning('auth backend overwrites auth.User object for %s '
                'with Active Directory data.' % username)
        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist:
            if ldap is None:
                return None
            user = None

        if not user:
            l = ldap.initialize(AD_LDAP_URL)
            l.simple_bind_s(username, password)
            result = l.search_ext_s(AD_SEARCH_DN, ldap.SCOPE_SUBTREE,
                    'sAMAccountName=%s' % username, AD_SEARCH_FIELDS)[0][1]
            l.unbind_s()

            # givenName == First Name
            if 'givenName' in result:
                first_name = result['givenName'][0]
            else:
                first_name = None

            # sn == Last Name (Surname)
            if 'sn' in result:
                last_name = result['sn'][0]
            else:
                last_name = None

            # mail == Email Address
            if 'mail' in result:
                email = result['mail'][0]
            else:
                email = None

            user = User(username=username, first_name=first_name,
                    last_name=last_name, email=email)
            user.is_staff = False
            user.is_superuser = False
            user.set_password(password)
            user.save()

        return user

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None

    def is_valid(self, username=None, password=None):
        ## Disallowing null or blank string as password
        ## as per comment: http://www.djangosnippets.org/snippets/501/#c868
        if not password or ldap is None:
            return False
        binddn = '%s@%s' % (username, AD_NT4_DOMAIN)
        try:
            l = ldap.initialize(AD_LDAP_URL)
            l.simple_bind_s(binddn, password)
            l.unbind_s()
            return True
        except ldap.INVALID_CREDENTIALS:
            logger.info('Active Directory authentication for \"%s\"" failed '
                    '(invalid credentials).', username)
            return False
        except ldap.LDAPError:
            logger.info('Active Directory authentication for \"%s\"" failed.',
                    username)
            return False



CACHE_KEY = getattr('settings', 'CACHE_MIDDLEWARE_KEY_PREFIX', '')
CACHE_GENERATION_KEY = CACHE_KEY + '/cache_gen'


def make_cache_key(key, user=None):
    from hashlib import md5
    gen = memcache.get(CACHE_GENERATION_KEY)
    if not gen:
        gen = 1
        memcache.set(CACHE_GENERATION_KEY, gen)
    from logging import info
    info("generation in model cache: %s", memcache.get(CACHE_GENERATION_KEY))
    return md5('%s/%d/%s' % (CACHE_KEY, gen, str(key))).hexdigest()


def invalidate_cache():
    gen = memcache.get(CACHE_GENERATION_KEY)
    if not gen:
        gen = 1
        memcache.set(CACHE_GENERATION_KEY, gen)
    else:
        memcache.incr(CACHE_GENERATION_KEY)
    from logging import info
    info("generation in model cache: %s", memcache.get(CACHE_GENERATION_KEY))


def invalidate_user_cache(user):
    pass



# number of bytes taken from /dev/urandom
RANDOM_ID_SOURCE_BYTES = 7

# from april fool's rfc 1924
BASE85 = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz' \
    '!#$%&()*+-;<=>?@^_`{|}~'

# rfc4648 alphabets
BASE16 = BASE85[:16]
BASE32 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ234567'
BASE32HEX = BASE85[:32]
BASE64 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'
BASE64URL = BASE64[:62] + '-_'

# http://en.wikipedia.org/wiki/Base_62 useful for url shorteners
BASE62 = BASE85[:62]


class NumConv(object):
    """ Class to create converter objects.

        :param radix: The base that will be used in the conversions.
           The default value is 10 for decimal conversions.
        :param alphabet: A string that will be used as a encoding alphabet.
           The length of the alphabet can be longer than the radix. In this
           case the alphabet will be internally truncated.

           The default value is :data:`numconv.BASE85`

        :raise TypeError: when *radix* isn't an integer
        :raise ValueError: when *radix* is invalid
        :raise ValueError: when *alphabet* has duplicated characters
    """

    def __init__(self, radix=10, alphabet=BASE85):
        if int(radix) != radix:
            raise TypeError('radix must be an integer')
        if not 2 <= radix <= len(alphabet):
            raise ValueError('radix must be >= 2 and <= %d' % (
                len(alphabet), ))
        self.radix = radix
        self.alphabet = alphabet
        self.cached_map = dict(zip(self.alphabet, range(len(self.alphabet))))
        if len(self.cached_map) != len(self.alphabet):
            raise ValueError("duplicate characters found in '%s'" % (
                self.alphabet, ))

    def int2str(self, num):
        """ Converts an integer into a string.

            Example usage:

            .. code-block:: python

               # 3735928559 to hexadecimal
               >> NumConv(16).int2str(3735928559)
               'DEADBEEF'

               # 19284 to binary
               >> NumConv(2).int2str(19284)
               '100101101010100'

               # 37 to base 4 using a custom dictionary
               >> NumConv(4, 'rofl').int2str(37)
               'foo'

               # Very large number to :data:`~numconv.BASE85`
               >> NumConv(85).int2str(2693233728041137L)
               '~123AFz@'

            Arguments:

            :param num: A numeric value to be converted to another base as a
                string.
            :returns: string
            :raise TypeError: when *num* isn't an integer
            :raise ValueError: when *num* isn't positive
        """
        if int(num) != num:
            raise TypeError('number must be an integer')
        if num < 0:
            raise ValueError('number must be positive')
        radix, alphabet = self.radix, self.alphabet
        if (radix in (8, 10, 16) and
                alphabet[:radix].lower() == BASE85[:radix].lower()):
            return ({8: '%o', 10: '%d', 16: '%x'}[radix] % num).upper()
        ret = ''
        while True:
            ret = alphabet[num % radix] + ret
            if num < radix:
                break
            num //= radix
        return ret

    def str2int(self, num):
        """ Converts a string into an integer.

            If possible, the built-in python conversion will be used for speed
            purposes.

            Example usage:

            .. code-block:: python

               # Hexadecimal 'DEADBEEF' to integer
              >> NumConv(16).str2int('DEADBEEF')
              3735928559L

               # Binary '100101101010100' to integer
               >> NumConv(2).str2int('100101101010100')
               19284

               # Base 4 with custom encoding 'foo' to integer
               >> NumConv(4, 'rofl').str2int('foo')
               37

               # :data:`~numconv.BASE85` '~123AFz@' to integer
               >> NumConv(85).str2int('~123AFz@')
               2693233728041137L

            :param num: A string that will be converted to an integer.
            :rtype: integer
            :raise ValueError: when *num* is invalid
        """
        radix, alphabet = self.radix, self.alphabet
        if radix <= 36 and alphabet[:radix].lower() == BASE85[:radix].lower():
            return int(num, radix)
        ret = 0
        lalphabet = alphabet[:radix]
        for char in num:
            if char not in lalphabet:
                raise ValueError("invalid literal for radix2int() with radix "
                    "%d: '%s'" % (radix, num))
            ret = ret * radix + self.cached_map[char]
        return ret


class Final(type):
    """ Metaclass for 'sealed' classes. A sealed class cannot be extended.
        Origin: BruceEckel / Python 3 Patterns & Idioms: Metaclasses
    """

    def __new__(cls, name, bases, classdict):
        for b in bases:
            if isinstance(b, Final):
                raise TypeError("type '%s' is not an acceptable base type" %
                    b.__name__)
        return type.__new__(cls, name, bases, dict(classdict))


def datetime_as_iso(value):
    """Helper function to format datetime object to ISO8601 string."""
    import datetime
    if not isinstance(value, datetime.datetime):
        return ""
    retval = value.strftime("%Y-%m-%dT%H:%M:%S%z")
    if not value.utcoffset():
        return "".join([retval, "Z"])
    return retval


def int2str(num, radix=10, alphabet=BASE85):
    """ Helper for quick base conversions from integers to strings """
    return NumConv(radix, alphabet).int2str(num)


def str2int(num, radix=10, alphabet=BASE85):
    """ Helper for quick base conversions from strings to integers. """
    return NumConv(radix, alphabet).str2int(num)


def clean_int(value, default, min_value=None, max_value=None):
    """ Helper to cast value to int and to clip it to min or max_value.

        Arguments:

        :param value: any value (preferably something that can be casted
            to ``int``).
        :param default: default value to be used when type casting fails.
        :param min_value: minimum allowed value.
        :param max_value: maximum allowed value.

        :returns:
            An integer between min_value and max_value.
    """
    if not isinstance(value, (int, long)):
        try:
            value = int(value)
        except (TypeError, ValueError):
            value = default
    if min_value is not None:
        value = max(min_value, value)
    if max_value is not None:
        value = min(value, max_value)
    return value


def str_to_bool(v):
    return v.strip().lower() in ("yes", "true", "on", "1")


def ip4_to_int(ip):
    """ Convert string contained valid IP4 address to 32-bit integer.
        >>> ip4_to_int("192.168.0.1")
        3232235521L
    """
    ip = ip.rstrip().split('.')
    ipn = 0
    while ip:
        ipn = (ipn << 8) + int(ip.pop(0))
    return ipn


def ip4_to_str(ip):
    """ Convert integer contained valid IP4 address to string.
        >>> ip4_to_str(3232235521)
        '192.168.0.1'
    """
    ips = ''
    for i in range(4):
        ip, n = divmod(ip, 256)
        ips = str(n) + '.' + ips
    return ips[:-1]  # take out extra point


def multi_split(s, seps):
    """ Split string by multiple separators.
        >>> multi_split("a,b;c:d", ",;:")
        ['a', 'b', 'c', 'd']
    """
    if not s or not seps:
        return [s]
    sep = seps[0]
    v = ''.join([ch if ch not in seps else sep for ch in s])
    return v.split(sep)


def _random(bytes):
    """ Get random bytes and convert them to integer. """
    import os
    table = ''.join(chr(i) for i in range(256))
    return str2int(os.urandom(bytes), 256, table)


def random_in_range(max_value, min_value=0):
    """ See How to generate a random number from within a range - C
    http://goo.gl/aWiZX
    """
    def numbits(x):
        """ Python before 2.7 does not have Long.bit_length method. """
        try:
            return x.bit_length()
        except AttributeError:
            return len(bin(abs(x))) - 2

    if not max_value:
        max_value = 2L ** 64 - 1
    bit_length = numbits(max_value)
    rand_max = 2L ** bit_length - 1
    import random
    r = random.SystemRandom()
    base_random = r.getrandbits(bit_length)

    # base_random in [0, RAND_MAX]
    if rand_max == base_random:
        return random_in_range(max_value=max_value, min_value=min_value)

    # now guaranteed to be in [0, RAND_MAX)
    value_range = max_value - min_value
    remainder = rand_max % value_range
    bucket = rand_max / value_range

    if base_random < rand_max - remainder:
        return min_value + base_random / bucket
    return random_in_range(max_value=max_value, min_value=min_value)


def get_random_id():
    """ Get random integer suitable for database ID. """
    return _random(RANDOM_ID_SOURCE_BYTES)


def get_random_id_str(alphabet=None):
    """ Get random integer and encode it to URL-safe string. """
    if not alphabet:
        alphabet = BASE62
    n = _random(RANDOM_ID_SOURCE_BYTES)
    return int2str(n, len(alphabet), alphabet)


def get_unique_id(is_unique=None):
    """ Get unique integer value suitable for use as a database key.

        :param is_unique: used to generate a bit shorter IDs than those
            based on UUID.  If this parameter is not specified UUID will be
            used as random data.
    """
    if not callable(is_unique):
        import uuid
        return uuid.uuid4().int
    else:
        id = get_random_id()
        while not is_unique(id):
            id = get_random_id()
        return id


def get_unique_id_str(is_unique=None, alphabet=None):
    """ Get unique string suitable for use as database key.

        :param is_unique: used to generate a bit shorter IDs than those
            based on UUID.  If this parameter is not specified UUID will be
            used as random data.
        :param alphabet: aplphabet used for generated value. Defaults to BASE62
    """
    if not alphabet:
        alphabet = BASE62
    if not callable(is_unique):
        import uuid
        return int2str(uuid.uuid4().int, len(alphabet), alphabet)
    else:
        id = get_random_id_str()
        while not is_unique(id):
            id = get_random_id_str(alphabet)
        return id


#
# cached_property
#

class _Missing(object):
    def __repr__(self):
        return 'no value'

    def __reduce__(self):
        return '_missing'

_missing = _Missing()


class cached_property(object):
    """ A decorator that converts a function into a lazy property.  The
        function wrapped is called the first time to retrieve the result
        and then that calculated result is used the next time you access
        the value.

        Example usage:

        .. code-block:: python

            class Foo(object):
                @cached_property
                def foo(self):
                    # calculate something important here
                    return 42

        The class has to have a `__dict__` in order for this property to
        work.
    """

    # implementation detail: this property is implemented as non-data
    # descriptor.  non-data descriptors are only invoked if there is
    # no entry with the same name in the instance's __dict__.
    # this allows us to completely get rid of the access function call
    # overhead.  If one choses to invoke __get__ by hand the property
    # will still work as expected because the lookup logic is replicated
    # in __get__ for manual invocation.

    def __init__(self, func, name=None, doc=None, writeable=False):
        self.__name__ = name or func.__name__
        self.__module__ = func.__module__
        self.__doc__ = doc or func.__doc__
        self.func = func

    def __get__(self, obj, type=None):
        if obj is None:
            return self
        value = obj.__dict__.get(self.__name__, _missing)
        if value is _missing:
            value = self.func(obj)
            obj.__dict__[self.__name__] = value
        return value


def monkeypatch_method(cls):
    """ Add the decorated method to the given class; replace as needed.

        If the named method already exists on the given class, it will
        be replaced, and a reference to the old method appended to a list
        at cls._old_<name>. If the "_old_<name>" attribute already exists
        and is not a list, KeyError is raised.

        Example usage::

            from <somewhere> import <someclass>

            @monkeypatch_method(<someclass>)
            def <newmethod>(self, args):
                return <whatever>

        This adds <newmethod> to <someclass>

        Origin: http://goo.gl/C1v0z
    """

    def decorator(func):
        fname = func.__name__

        old_func = getattr(cls, fname, None)
        if old_func is not None:
            # Add the old func to a list of old funcs.
            old_ref = "_old_%s" % fname
            old_funcs = getattr(cls, old_ref, None)
            if old_funcs is None:
                setattr(cls, old_ref, [])
            elif not isinstance(old_funcs, list):
                raise KeyError("%s.%s already exists." %
                    (cls.__name__, old_ref))
            getattr(cls, old_ref).append(old_func)

        setattr(cls, fname, func)
        return func

    return decorator


if __name__ == "__main__":
    import doctest
    doctest.testmod()



from functools import wraps

from django.conf import settings
from django.http import Http404, HttpResponse, HttpResponseForbidden
from django.utils import simplejson

__all__ = (
    'ajax_only', 'ajax_login_required', 'ajax_template', 'auth_user_only',
    'debug_only', 'render_to_json_response', 'super_user_only',
)


_JSON_MIME_TYPE = 'application/json'


def ajax_login_required(view_func):
    def wrap(request, *args, **kwargs):
        if request.user.is_authenticated():
            return view_func(request, *args, **kwargs)
        else:
            json = simplejson.dumps({'login_required': True})
            return HttpResponseForbidden(json, mimetype=_JSON_MIME_TYPE)
    return wrap


def ajax_only(view_func):
    """ Ensure that all requests made to a view are made as AJAX requests.
        Non-AJAX requests will recieve a 400 (Bad Request) response.
    """
    @wraps(view_func)
    def wrapper(request, *args, **kwargs):
        if not request.is_ajax():
            from django.http import HttpResponseBadResponse
            return HttpResponseBadResponse()
        return view_func(request, *args, **kwargs)
    return wrapper


def ajax_template(template_name):
    def internal(view_func):
        def wrap(request, *args, **kwargs):
            if request.is_ajax() and template_name:
                kwargs.update({'template_name': template_name})
            return view_func(request, *args, **kwargs)
        return wrap
    return internal


def auth_user_only(view_func):
    def wrap(request, *args, **kwargs):
        if request.user.is_authenticated():
            return view_func(request, *args, **kwargs)
        else:
            raise Http404
    return wrap


def debug_only(handler):
    def wrap(request, *args, **kw):
        if not (settings.DEBUG or getattr(request.user, 'is_superuser', None)):
            raise Http404
        return handler(request, *args, **kw)
    wrap.__name__ = handler.__name__
    return wrap


def message_if(test_func, message):
    """ Drop a message before view_func run if test_func returns True. """
    def internal(view_func):
        def wrap(request, *args, **kwargs):
            if test_func(request):
                from django.contrib import messages
                if callable(message):
                    text = message(request)
                else:
                    text = message
                if text:
                    messages.warning(request, text)
            return view_func(request, *args, **kwargs)
        return wrap
    return internal


def render_to_json_response(*fn, **jsonargs):
    """ Render response as JSON.

        :param encoder: custom `JSONEncoder` subclass to serialize additional
            types. This argument is an alias to `cls` argument for `json.dump`.
        :param jsonargs: arguments suitable to pass to `json.dump` function.
        :returns: `HttpResponse` object with JSON mime type.

        Usage::

            @render_to_json_response
            def json_view(request):
                return { 'foo': 'bar' }

            @render_to_json_response(indent=4)
            def json_view(request):
                return { 'foo': 'bar' }
    """
    def internal(view_func):
        @wraps(view_func)
        def wrapper(request, *args, **kwargs):
            response = view_func(request, *args, **kwargs) or {}
            status_code = kwargs.pop('status', None)
            if isinstance(response, HttpResponse):
                response.mimetype = _JSON_MIME_TYPE
                if status_code:
                    response.status_code = status_code
                return response

            ret = HttpResponse(mimetype=_JSON_MIME_TYPE)
            if status_code:
                ret.status_code = status_code
            encoder = jsonargs.pop('encoder', None)
            ret.write(simplejson.dumps(response, cls=encoder, **jsonargs))
            return ret
        return wrapper
    if len(fn) > 0 and callable(fn[0]):
        return internal(fn[0])
    return internal


def super_user_only(view_func):
    def wrap(request, *args, **kwargs):
        if request.user.is_authenticated() and request.user.is_superuser:
            return view_func(request, *args, **kwargs)
        else:
            raise Http404
    return wrap



from __future__ import unicode_literals
from django.db import models
from django.utils import six
from mcutils.django import model_uid_generator


class UidIntegerField(models.BigIntegerField):
    def __init__(self, model_class, **kwargs):
        self.model_class = model_class
        self.min_value = kwargs.pop('min_value')
        self.max_value = kwargs.pop('max_value', self.MAX_BIGINT)
        self.field_name = kwargs.pop('field_name', 'pk')
        kwargs.setdefault('unique', True)
        kwargs.setdefault('editable', False)
        kwargs.setdefault('default', model_uid_generator(
            self.model_class, max_value=self.max_value,
            min_value=self.min_value, field_name=self.field_name))
        super(UidIntegerField, self).__init__(**kwargs)

    def value_to_string(self, obj):
        return six.text_type(self._get_val_from_obj(obj)).zfill(
            len(six.text_type(self.max_value)))

    def deconstruct(self):
        name, path, args, kwargs = super(UidIntegerField, self).deconstruct()
        model_class = self.model_class
        if isinstance(model_class, models.base.ModelBase):
            model_class = '.'.join([
                model_class._meta.app_label, model_class.__class__.__name__])
        args = [six.text_type(model_class)]
        kwargs['min_value'] = self.min_value
        if self.max_value != self.MAX_BIGINT:
            kwargs['max_value'] = self.max_value
        if self.field_name != 'pk':
            kwargs['field_name'] = six.text_type(self.field_name)
        default = kwargs.get('default')
        if callable(default):
            kwargs['default'] = default()
        return name, path, args, kwargs

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([
        ([UidIntegerField], [], {
            'model_class': ['model_class', {}],
            'field_name': ['field_name', {'default': 'pk'}],
            'min_value': ['min_value', {'default': 0}],
        })
    ], ['^utils\.django\.fields\.UidIntegerField'])
except ImportError:
    add_introspection_rules = lambda x: None



# -*- coding: utf-8 -*-
import django
import django.template
import django.utils.safestring
from django.conf import settings
from django.utils import six
from .collections_utils import in_list
from .form_utils import fieldset
from .http_utils import url_name


def assignment_tag(self, func=None, takes_context=None):
    from inspect import getargspec
    from django.template import TemplateSyntaxError

    def dec(func):
        params, xx, xxx, defaults = getargspec(func)
        if takes_context:
            if params[0] == 'context':
                params = params[1:]
            else:
                raise TemplateSyntaxError("Any tag function decorated with "
                    "takes_context=True must have a first argument "
                    "of 'context'")

        class AssignmentNode(django.template.Node):
            def __init__(self, params_vars, target_var):
                self.params_vars = map(django.template.Variable, params_vars)
                self.target_var = target_var

            def render(self, context):
                resolved_vars = [var.resolve(context) for var in
                    self.params_vars]
                if takes_context:
                    func_args = [context] + resolved_vars
                else:
                    func_args = resolved_vars
                context[self.target_var] = func(*func_args)
                return ''

        def compile_func(parser, token):
            bits = token.split_contents()
            tag_name = bits[0]
            bits = bits[1:]
            params_max = len(params)
            defaults_length = defaults and len(defaults) or 0
            params_min = params_max - defaults_length
            if (len(bits) < 2 or bits[-2] != 'as'):
                raise TemplateSyntaxError(
                    "'%s' tag takes at least 2 arguments and the "
                    "second last argument must be 'as'" % tag_name)
            params_vars = bits[:-2]
            target_var = bits[-1]
            if (len(params_vars) < params_min or
                    len(params_vars) > params_max):
                if params_min == params_max:
                    raise TemplateSyntaxError(
                        "%s takes %s arguments" % (tag_name, params_min))
                else:
                    raise TemplateSyntaxError(
                        "%s takes between %s and %s arguments"
                        % (tag_name, params_min, params_max))
            return AssignmentNode(params_vars, target_var)

        compile_func.__doc__ = func.__doc__
        self.tag(getattr(func, "_decorated_function", func).__name__,
            compile_func)
        return func

    if func is None:
        # @register.assignment_tag(...)
        return dec
    elif callable(func):
        # @register.assignment_tag
        return dec(func)
    else:
        raise TemplateSyntaxError("Invalid arguments provided to "
            "assignment_tag")


if django.VERSION < (1, 4, None):
    import utils
    utils.monkeypatch_method(django.template.Library)(assignment_tag)

register = django.template.Library()


class VerbatimNode(django.template.Node):
    def __init__(self, text):
        self.text = text

    def render(self, context):
        return self.text


def do_verbatim(parser, token):
    text = []
    while 1:
        token = parser.tokens.pop(0)
        if token.contents == 'endverbatim':
            break
        if token.token_type == django.template.TOKEN_VAR:
            text.append('{{')
        elif token.token_type == django.template.TOKEN_BLOCK:
            text.append('{%')
        text.append(token.contents)
        if token.token_type == django.template.TOKEN_VAR:
            text.append('}}')
        elif token.token_type == django.template.TOKEN_BLOCK:
            text.append('%}')
    return VerbatimNode(''.join(text))


if django.VERSION < (1, 5, None):
    register.tag("verbatim")(do_verbatim)


@register.filter
def add_url_parameters(value, arg):
    """ Add GET parameters to URL string.
        Usage:
            {{ request.get_full_path|add_url_parameters:"mode=list&limit=9" }}
    """
    if not isinstance(arg, basestring):
        arg = unicode(arg)
    from django.http import QueryDict, parse_qsl
    parts = value.split('?')
    path = parts[0]
    q = '&'.join(parts[1:])
    args = QueryDict(q).copy()
    new_args = parse_qsl(arg or '')
    for (k, v) in new_args:
        args.update({k: v})
    s = lambda x, y: "%s=%s" % (x, y) if y else "%s" % x
    q = '&'.join([s(k, v) for (k, v) in args.items()])
    return '?'.join([path, q])


register.filter('in_list', in_list)


@register.filter
def escapejson(value, arg=None):
    import simplejson
    base = simplejson.dumps(value)
    return base.replace('/', r'\/')


register.assignment_tag(fieldset)


DEFAULT_WINDOW = getattr(settings, 'PAGINATION_DEFAULT_WINDOW', 4)


@register.inclusion_tag("paginator.html", takes_context=True)
def paginator(context, window=DEFAULT_WINDOW):
    """ Renders the ``pagination/pagination.html`` template, resulting in a
    Digg-like display of the available pages, given the current page.  If there
    are too many pages to be displayed before and after the current page, then
    elipses will be used to indicate the undisplayed gap between page numbers.

    Requires one argument, ``context``, which should be a dictionary-like data
    structure and must contain the following keys:

    ``paginator``
        A ``Paginator`` or ``QuerySetPaginator`` object.

    ``page_obj``
        This should be the result of calling the page method on the
        aforementioned ``Paginator`` or ``QuerySetPaginator`` object, given
        the current page.

    This same ``context`` dictionary-like data structure may also include:

    ``getvars``
        A dictionary of all of the **GET** parameters in the current request.
        This is useful to maintain certain types of state, even when requesting
        a different page.
        """
    try:
        paginator = context['paginator']
        page_obj = context['page_obj']
        page_range = paginator.page_range
        # First and last are simply the first *n* pages and the last *n* pages,
        # where *n* is the current window size.
        first = set(page_range[:window])
        last = set(page_range[-window:])
        # Now we look around our current page, making sure that we don't wrap
        # around.
        current_start = page_obj.number - 1 - window
        if current_start < 0:
            current_start = 0
        current_end = page_obj.number - 1 + window
        if current_end < 0:
            current_end = 0
        current = set(page_range[current_start:current_end])
        pages = []
        # If there's no overlap between the first set of pages and the current
        # set of pages, then there's a possible need for elusion.
        if len(first.intersection(current)) == 0:
            first_list = list(first)
            first_list.sort()
            second_list = list(current)
            second_list.sort()
            pages.extend(first_list)
            diff = second_list[0] - first_list[-1]
            # If there is a gap of two, between the last page of the first
            # set and the first page of the current set, then we're missing a
            # page.
            if diff == 2:
                pages.append(second_list[0] - 1)
            # If the difference is just one, then there's nothing to be done,
            # as the pages need no elusion and are correct.
            elif diff == 1:
                pass
            # Otherwise, there's a bigger gap which needs to be signaled for
            # elusion, by pushing a None value to the page list.
            else:
                pages.append(None)
            pages.extend(second_list)
        else:
            unioned = list(first.union(current))
            unioned.sort()
            pages.extend(unioned)
        # If there's no overlap between the current set of pages and the last
        # set of pages, then there's a possible need for elusion.
        if len(current.intersection(last)) == 0:
            second_list = list(last)
            second_list.sort()
            diff = second_list[0] - pages[-1]
            # If there is a gap of two, between the last page of the current
            # set and the first page of the last set, then we're missing a
            # page.
            if diff == 2:
                pages.append(second_list[0] - 1)
            # If the difference is just one, then there's nothing to be done,
            # as the pages need no elusion and are correct.
            elif diff == 1:
                pass
            # Otherwise, there's a bigger gap which needs to be signaled for
            # elusion, by pushing a None value to the page list.
            else:
                pages.append(None)
            pages.extend(second_list)
        else:
            differenced = list(last.difference(current))
            differenced.sort()
            pages.extend(differenced)
        to_return = {
            'pages': pages,
            'page_obj': page_obj,
            'paginator': paginator,
            'is_paginated': paginator.count > paginator.per_page,
        }
        if 'request' in context:
            getvars = context['request'].GET.copy()
            if 'page' in getvars:
                del getvars['page']
            if len(getvars.keys()) > 0:
                to_return['getvars'] = "&%s" % getvars.urlencode()
            else:
                to_return['getvars'] = ''
        return to_return
    except KeyError, AttributeError:
        return {}


class SiteNameNode(django.template.Node):

    def __init__(self, site=None, var_name=None):
        from django.contrib.sites.models import Site
        if isinstance(site, Site):
            self.site = site
        else:
            self.site = django.template.Variable(site)
        self.var_name = var_name

    def render(self, context):
        from django.contrib.sites.models import Site
        try:
            if isinstance(self.site, Site):
                site = self.site
            else:
                site = self.site.resolve(context)
            site_name = site.name or ''
            if self.var_name:
                context[self.var_name] = site_name
                return ''
            else:
                return site_name
        except django.template.VariableDoesNotExist:
            return ''


@register.tag()
def site_name(parser, token):
    """ site title.
        {% site_name %}
        {% site_name as title %}
        {% site_name request.site %}
        {% site_name request.site as title %}
    """
    from django.contrib.sites.models import Site
    current_site = Site.objects.get_current()
    try:
        args = token.split_contents()
        var_name = None
        if len(args) == 1:
            # {% site_name %} case
            arg = current_site
        elif len(args) == 2:
            # {% site_name site %} case
            tag_name, arg = args
        elif len(args) == 3:
            # {% site_name as var %} case
            arg = current_site
            tag_name, _as, var_name = args
        elif len(args) == 4:
            # {% site_name site as var %} case
            tag_name, arg, _as, var_name = args
    except ValueError:
        raise django.template.TemplateSyntaxError, \
            "%r tag requeres arguments" % token.contents.split()[0]
    return SiteNameNode(arg, var_name)


@register.assignment_tag(name='type_label')
def type_label_tag(val):
    return type(val).__name__.lower()


@register.filter(name='type_label')
def type_label_filter(val):
    return type(val).__name__.lower()


register.assignment_tag(url_name, takes_context=True)



__all__ = ['crc16', 'get_hexdigest', 'luhn']


def crc16(buff, crc=0, poly=0xa001):
    l = len(buff)
    i = 0
    while i < l:
        ch = ord(buff[i])
        uc = 0
        while uc < 8:
            if (crc & 1) ^ (ch & 1):
                crc = (crc >> 1) ^ poly
            else:
                crc >>= 1
            ch >>= 1
            uc += 1
        i += 1
    return crc


def get_hexdigest(algorithm, salt, s):
    """ Returns a string of the hexdigest of the given string and salt
    using the given algorithm ('md5', 'sha1' or 'crypt').
    """
    import sys
    if sys.version_info >= (2, 5):
        import hashlib
        md5_constructor = hashlib.md5
        md5_hmac = md5_constructor
        sha_constructor = hashlib.sha1
        sha_hmac = sha_constructor
    else:
        import md5
        md5_constructor = md5.new
        md5_hmac = md5
        import sha
        sha_constructor = sha.new
        sha_hmac = sha

    if algorithm == 'crypt':
        try:
            import crypt
        except ImportError:
            raise ValueError('"crypt" algorithm not supported in this '
                'environment')
        return crypt.crypt(s, salt)

    if algorithm == 'md5':
        return md5_constructor(salt + s).hexdigest()
    elif algorithm == 'sha1':
        return sha_constructor(salt + s).hexdigest()
    raise ValueError("Got unknown algorithm type.")


LUHN_ODD_LOOKUP = (0, 2, 4, 6, 8, 1, 3, 5, 7, 9)  # sum_of_digits(index * 2)


def luhn(candidate):
    """ Checks a candidate number for validity according to the Luhn
    algorithm (used in validation of, for example, credit cards).
    Both numeric and string candidates are accepted.
    """
    if not isinstance(candidate, basestring):
        candidate = str(candidate)
    try:
        evens = sum([int(c) for c in candidate[-1::-2]])
        odds = sum([LUHN_ODD_LOOKUP[int(c)] for c in candidate[-2::-2]])
        return ((evens + odds) % 10 == 0)
    except ValueError:  # Raised if an int conversion fails
        return False



def reset_queries():
    from django.db import reset_queries, connection
    result = len(connection.queries), sum(
        map(lambda q: float(q['time']), connection.queries))
    reset_queries()
    return result



from django.db import models
from django.core.management import call_command


def load_fixture(file_name, orm):
    """ Proper fixtures loading in south data migrations (to be ensured that
    migration will use appropriate version of the models for fixture loading

    https://djangosnippets.org/snippets/2897/

    Usage::

        def forwards(self, orm):
             load_fixture('my_fixture.json', orm)
    """
    original_get_model = models.get_model

    def get_model_southern_style(*args):
        try:
            return orm['.'.join(args)]
        except IndexError:
            return original_get_model(*args)

    models.get_model = get_model_southern_style
    call_command('loaddata', file_name)
    models.get_model = original_get_model



import copy

from django.template import Library
from django.utils import six
from django.utils.datastructures import SortedDict


register = Library()


@register.assignment_tag
def fieldset(form, field_names):
    fields = list(filter(lambda s: bool(s.strip()), field_names.split(',')))
    new_form = copy.copy(form)
    new_form.fields = SortedDict(
        [(k, v) for k, v in six.iteritems(form.fields) if k in fields])
    return new_form


@register.inclusion_tag('include/form.html')
def render_form(form, formsets=None):
    return {
        'form': form,
    }



import re
from django.conf import settings
from django.contrib.auth.decorators import login_required


class RequireLoginMiddleware(object):
    """ Middleware component that wraps the login_required decorator around
        matching URL patterns. To use, add the class to MIDDLEWARE_CLASSES and
        define LOGIN_REQUIRED_URLS and LOGIN_REQUIRED_URLS_EXCEPTIONS in your
        settings.py. For example:
        ------
        LOGIN_REQUIRED_URLS = (
            r'/topsecret/(.*)$',
        )
        LOGIN_REQUIRED_URLS_EXCEPTIONS = (
            r'/topsecret/login(.*)$',
            r'/topsecret/logout(.*)$',
        )
        ------
        LOGIN_REQUIRED_URLS is where you define URL patterns; each pattern must
        be a valid regex.

        LOGIN_REQUIRED_URLS_EXCEPTIONS is, conversely, where you explicitly
        define any exceptions (like login and logout URLs).

        Code comes from http://djangosnippets.org/snippets/1220/
    """
    def __init__(self):
        self.required = tuple([re.compile(url) for url in getattr(settings,
            'LOGIN_REQUIRED_URLS', tuple())])
        self.exceptions = tuple([re.compile(url) for url in  getattr(settings,
            'LOGIN_REQUIRED_URLS_EXCEPTIONS', tuple())])

    def process_view(self,request,view_func,view_args,view_kwargs):
        # No need to process URLs if user already logged in
        if request.user.is_authenticated(): return None
        # An exception match should immediately return None
        for url in self.exceptions:
            if url.match(request.path):
                return None
        # Requests matching a restricted URL pattern are returned
        # wrapped with the login_required decorator
        for url in self.required:
            if url.match(request.path):
                return login_required(view_func)(request,*view_args,**view_kwargs)
        # Explicitly return None for all non-matching requests
        return None



import functools
import logging
from django.core import cache
from django.utils import timezone


logger = logging.getLogger(__name__)


class MutexError(Exception):
    pass


class mutually_exclusive(object):
    def __init__(self, lock_id, timeout=None, cache_alias=None,
                 fail_silently=False):
        self.lock_id = lock_id
        self.fail_silently = fail_silently
        self.timeout = timeout
        self.cache = cache.get_cache(cache_alias or cache.DEFAULT_CACHE_ALIAS)

    def __call__(self, func):
        return self.decorate_callable(func)

    def __enter__(self):
        if self.cache.has_key(self.lock_id):
            raise MutexError('Could not acquire lock: {0}'.format(self.lock_id))
        self.cache.add(self.lock_id, timezone.now(), self.timeout)

    def __exit__(self, *args):
        self.cache.delete(self.lock_id)

    def decorate_callable(self, func):
        """ Decorates a function with the mutex decorator by using this class
        as a context manager around it.
        """
        def wrapper(*args, **kwargs):
            try:
                with self:
                    return func(*args, **kwargs)
            except MutexError as e:
                logger.warn(e.message)
                if not self.fail_silently:
                    raise e

        functools.update_wrapper(wrapper, func)
        return wrapper



#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
from setuptools import setup, find_packages


setup(name='metacorus-django-utils',
    version='1.1.4',
    author='S.Prymak',
    author_email='sprymak@metacorus.com',
    url='https://github.com/sprymak/metacorus-django-utils',
    license='LICENSE',
    packages=find_packages(),
    install_requires=[
        "html5lib",
        "unidecode",
    ],
)



from django.conf import settings
from django.contrib import auth
from django.contrib.auth.backends import ModelBackend
from django.db.models import Q


try:
    from django.contrib.auth import get_user_model
except ImportError:
    import django.contrib.auth.models
    get_user_model = lambda: django.contrib.auth.models.User


SPLIT_CHAR = getattr(settings, 'AUTH_SUBSTUSER_SPLIT_CHAR', '?')


class SubstUserBackend(ModelBackend):
    """ Let superusers login as regular users.
    http://nedbatchelder.com/blog/201008/django_superuser_login_trapdoor.html
    """
    def authenticate(self, username=None, password=None, **kwargs):
        # The password should be name/password
        if SPLIT_CHAR not in password:
            return None

        user_model = get_user_model()
        username_field = getattr(user_model, 'USERNAME_FIELD', 'username')
        if username is None:
            username = kwargs.get(username_field)
        if username is None:
            return
        try:
            user = user_model._default_manager.get(
                Q(username__iexact=username) | Q(email__iexact=username))
        except (user_model.DoesNotExist, user_model.MultipleObjectsReturned):
            return None

        # authenticate superuser with passed credentials
        username, password = password.split(SPLIT_CHAR, 1)
        credentials = {username_field: username, 'password': password}
        superuser = auth.authenticate(**credentials)
        if superuser and superuser.is_superuser and superuser.is_active:
            return user



import os
from urllib import quote
from urlparse import urljoin


def get_host_name():
    host_name = os.environ.get('HTTP_HOST', os.environ.get('SERVER_NAME', ''))
    if ':' in host_name:
        host_name = host_name[:host_name.find(':')]
    return host_name


def get_host_url():
    protocol = os.environ.get('wsgi.url_scheme')
    if not protocol:
        if os.environ.get('HTTPS') == 'on':
            protocol = 'https'
        else:
            protocol = 'http'
    hostname = os.environ.get(
        'HTTP_HOST', os.environ.get('SERVER_NAME', 'localhost'))
    port = os.environ.get('SERVER_PORT', '')

    url = '://'.join([protocol, hostname])
    if port and not ':' in hostname:
        if protocol == 'https' and port != '443':
            url += ':' + port
        if protocol == 'http' and port != '80':
            url += ':' + port
    return url


def build_absolute_uri(location):
    """ Returns the absolute URI form of location. """
    return urljoin(get_host_url(), quote(location))


def get_dict(post, key):
    """ Extract from POST PHP-like arrays as dictionary.

        Example usage::

            <input type="text" name="option[key1]" value="Val 1">
            <input type="text" name="option[key2]" value="Val 2">

            options = get_dict(request.POST, 'option')
            options['key1'] is 'Val 1'
            options['key2'] is 'Val 2'
    """
    result = {}
    if post:
        import re
        patt = re.compile('^([a-zA-Z_]\w+)\[([a-zA-Z_\-][\w\-]*)\]$')
        for post_name, value in post.items():
            value = post[post_name]
            match = patt.match(post_name)
            if not match or not value:
                continue
            name = match.group(1)
            if name == key:
                k = match.group(2)
                result.update({k: value})
    return result
